/* global HTMLElement, customElements */

// HTML Example (PUG):
// wandering-teaser-image(img="#some-image-inside")
//   img#some-image-inside(src="./my-image.png")
class WanderingTeaserImage extends HTMLElement {
  constructor () {
    super();
    this.adjustmentSpeedFactor = 0.5;
    this.updating = false;
  }

  connectedCallback () {
    try {
      this.image = this.querySelector(this.imgSelector);

      this.viewChangeListener = _ => this.onViewChange();
      window.addEventListener('scroll', this.viewChangeListener);
      window.addEventListener('resize', this.viewChangeListener);
      this.updateImagePosition();
    } catch (e) {
      console.warn(e.message);
    }
  }

  disconnectedCallback () {
    window.removeEventListener('scroll', this.viewChangeListener);
    window.removeEventListener('resize', this.viewChangeListener);
  }

  onViewChange () {
    if (!this.updating) {
      this.updating = true;
      window.requestAnimationFrame(() => this.updateImagePosition());
    }
  }

  updateImagePosition () {
    const { top: containerTop } = this.getBoundingClientRect();
    const { height: imageHeight } = this.image.getBoundingClientRect();

    const windowWidthBasedModifier = Math.min(1 + window.innerWidth / 1000, 2);

    const imageTopAdjustment = Math.min(-1 * this.adjustmentSpeedFactor * containerTop, imageHeight / windowWidthBasedModifier);
    this.image.style.bottom = `${-1 * imageTopAdjustment}px`;
    this.updating = false;
  }

  get imgSelector () {
    if (!this.hasAttribute('img')) {
      throw new Error('WanderingTeaserImage cannot be established: img target selector is not defined!')
    }
    return this.getAttribute('img')
  }
}

customElements.define('wandering-teaser-image', WanderingTeaserImage);

/* global HTMLElement, customElements, IntersectionObserver */
class ScrollFoam extends HTMLElement {
  connectedCallback () {
    this.mainElement = this.querySelector('main');
    if (typeof IntersectionObserver !== 'undefined') {
      this.observer = new IntersectionObserver((entries) => {
      // isIntersecting is true when element and viewport are overlapping
      // isIntersecting is false when element and viewport don't overlap
        if (entries[0].isIntersecting === true) {
          this.classList.remove('squeeze-out');
          this.classList.add('squeeze');
        } else {
          this.classList.add('squeeze-out');
        }
      }, { threshold: [0] });
    }

    this.observer.observe(this.querySelector('footer'));
  }

  disconnectedCallback () {
    this.observer?.disconnect();
  }
}

customElements.define('scroll-foam', ScrollFoam);

/* global customElements, HTMLButtonElement, CustomEvent */

const eventOptionDefaults = {
  bubbles: true
};
class NavButton extends HTMLButtonElement {
  constructor () {
    super();
    this.active = false;
  }

  connectedCallback () {
    this.nav = this.parentElement;

    this.clickListener = () => {
      this.active ? this.deactivate() : this.activate();
    };
    this.addEventListener('click', this.clickListener);
  }

  disconnectedCallback () {
    this.removeEventListener('click', this.clickListener);
  }

  activate () {
    this.active = true;
    this.dispatchEvent(new CustomEvent('nav-button:activated', eventOptionDefaults));
    this.classList.add('active');
    this.nav.classList.add('open');
  }

  deactivate () {
    this.active = false;
    this.dispatchEvent(new CustomEvent('nav-button:deactivated', eventOptionDefaults));
    this.classList.remove('active');
    this.nav.classList.remove('open');
  }
}

customElements.define('nav-button', NavButton, { extends: 'button' });

/* global HTMLElement, customElements, IntersectionObserver */
class TopPageMenu extends HTMLElement {
  constructor () {
    super();

    const options = {
      root: null,
      // rootMargin: '0px',
      threshold: 0.2
    };

    if (('IntersectionObserver' in window)) {
      this.observer = new IntersectionObserver((entries) => this.isVisible(entries), options);
    }
  }

  async connectedCallback () {
    const firstMeaningfulContent = document.querySelector('scroll-foam main');
    this.observer?.observe(firstMeaningfulContent);
  }

  disconnectedCallback () {
    this.observer?.disconnect();
  }

  isVisible (entries) {
    if (entries[0].intersectionRatio < 0.2) {
      // console.log('does not intersect')
      document.body?.classList.remove('main-content-visible');
    } else {
      // console.log('intersects')
      document.body?.classList.add('main-content-visible');
    }
  }
}

customElements.define('top-page-menu', TopPageMenu);

/* global HTMLElement, customElements */

class ImageGallery extends HTMLElement {
  #dialog
  #imageContainer
  #nav
  #next
  #close
  #download
  #previous
  #currentImage
  #currentImageIndex = 0
  #outsideClickHandler = (e) => {
    if(e.target === e.currentTarget) this.onclose();
    // console.log('outside click')
    // console.log(e.target, e.currentTarget)
  }

  #thumbnailClickHandler = (imageLink, index) => async (e) => {
    e.preventDefault();
    await this.onThumbnailClick(imageLink, index);
    return false
  }

  connectedCallback () {
    this.#dialog = dialogElement();
    this.#imageContainer = imgWrapperElement();
    this.#dialog.appendChild(this.#imageContainer);

    this.#nav = buttonNav();
    this.#next = selectNextButtonElement();
    this.#previous = selectPreviousButtonElement();

    this.#nav.appendChild(this.#previous);
    this.#nav.appendChild(this.#next);
    this.#dialog.appendChild(this.#nav);

    this.#close = closeButtonElement();
    this.#dialog.appendChild(this.#close);
    this.appendChild(this.#dialog);

    this.imagesLinks = Array.from(this.querySelectorAll('image-gallery-thumbnail a'));

    this.imagesLinks.forEach((imageLink, index) => {
      imageLink.addEventListener('click', this.#thumbnailClickHandler(imageLink, index));
    });

    this.#next.addEventListener('click', this.onNextClick.bind(this));
    this.#previous.addEventListener('click', this.onPreviousClick.bind(this));

    this.#dialog.addEventListener('close', () => {
      this.#imageContainer.innerHTML = '';
    });

    this.#close.addEventListener('click', this.onclose.bind(this));
  }

  onclose() {
    this.#download?.remove();
    this.#dialog?.close();
    this.#dialog?.removeEventListener('click', this.#outsideClickHandler);
  }

  async onNextClick() {
    this.#currentImageIndex = (this.#currentImageIndex + 1) % this.imagesLinks.length;
    console.log(this.#currentImageIndex, this.imagesLinks.length);
    const nextImageLink = this.imagesLinks.at(this.#currentImageIndex);
    if (nextImageLink) {
      await this.changeImage(nextImageLink);
    }
  }

  async onPreviousClick() {
    this.#currentImageIndex = this.#currentImageIndex - 1 >= 0 ? this.#currentImageIndex - 1 : this.imagesLinks.length - 1;
    console.log(this.#currentImageIndex);
    const previousImageLink = this.imagesLinks.at(this.#currentImageIndex);
    if (previousImageLink) {
      await this.changeImage(previousImageLink);
    }
  }

  async onThumbnailClick(imageLink, index) {
    this.#currentImageIndex = index;
    await this.changeImage(imageLink);
    await this.#dialog?.showModal();

    this.#dialog?.addEventListener('click', this.#outsideClickHandler);
  }

  async changeImage(imageLink) {
    this.#currentImage = imageElement(imageLink);

    await this.#download?.remove();
    this.#download = await downloadLinkElement(imageLink);
    this.#dialog.appendChild(this.#download);

    if  (this.#dialog){
      this.#imageContainer.innerHTML = '';
      await this.#imageContainer.appendChild(this.#currentImage);
    }
  }
}

customElements.define('image-gallery', ImageGallery);


function dialogElement(){
  const element = document.createElement('DIALOG');
  element.classList.add('image-gallery-dialog');
  return element
}

function imgWrapperElement() {
  const element = document.createElement('DIV');
  element.classList.add('image-gallery-img-wrapper');
  return element
}

function imageElement(image) {
  const element = document.createElement('IMG');
  element.src = image.href;
  element.alt = image.title;
  return element
}

function selectNextButtonElement(){
  return selectSiblingButton('next')
}

function selectPreviousButtonElement(){
  return selectSiblingButton('previous')
}

function buttonNav() {
  const element = document.createElement('DIV');
  element.classList.add('image-gallery-nav');
  return element
}

function selectSiblingButton(direction){
  if (!['next', 'previous'].includes(direction)) throw new Error('Invalid direction')

  const element = document.createElement('BUTTON');
  element.setAttribute('type', 'button');
  element.setAttribute('aria-label', direction);
  element.classList.add('image-gallery-select-sibling');
  element.classList.add(`image-gallery-select-${direction}`);
  return element
}

function closeButtonElement(){
  const element = document.createElement('BUTTON');
  element.setAttribute('type', 'button');
  element.setAttribute('aria-label', 'close');
  element.classList.add('image-gallery-close');
  return element
}

async function downloadLinkElement(imageLink) {
  const res = await fetch(imageLink.href);
  let url;
  if (!res.ok) {url = imageLink.href; } // fallback to original url
  const file = await res.blob();
  url = URL.createObjectURL(file);
  const element = document.createElement('a');
  element.setAttribute('aria-label', 'download');
  element.setAttribute('target', '_blank');
  element.setAttribute('rel', 'noopener');
  element.href = url;
  element.setAttribute('download', imgFileName(imageLink));
  element.classList.add('download-image');
  return element
}

function imgFileName({ href, title }) {
  return href.split('/').pop()
}

/* global HTMLElement, customElements */
class LocationMap extends HTMLElement {
  #zoom = 100
  #zoomFactor = 50
  #resizeDebounce = null
  #inResize = false

  connectedCallback () {
    this.zoomInButton = zoomButton('in');
    this.zoomOutButton = zoomButton('out');
    const actionWrapperElement = actionWrapper();

    actionWrapperElement.appendChild(this.zoomInButton);
    actionWrapperElement.appendChild(this.zoomOutButton);
    this.zoomInButton.addEventListener('click', this.zoomIn.bind(this));
    this.zoomOutButton.addEventListener('click', this.zoomOut.bind(this));
    this.zoomOutButton.disabled = true;
    this.appendChild(actionWrapperElement);

    window.addEventListener('resize', this.resizeListener.bind(this));
  }


  resizeListener() {
    if (!this.#inResize) {
      this.classList.add('resizing');
      this.#inResize = true;
    }

    clearTimeout(this.#resizeDebounce);

    this.#resizeDebounce = setTimeout(() => {
      this.#inResize = false;
      this.classList.remove('resizing');
    }, 200);

  }

  hideWhileResizing() {
    this.classList.add('resizing');
    return this.resizeListener.bind(this)
  }

  zoomIn() {
    this.#zoom += this.#zoomFactor;
    if (this.#zoom > 250) { this.zoomInButton.disabled = true; }
     if(this.#zoom > 100) { this.zoomOutButton.disabled = false; }
    window.requestAnimationFrame(() => {
      this.style.setProperty('--location-zoom', `${this.#zoom}%`);
    });
  }

  zoomOut() {
    this.#zoom -= this.#zoomFactor;
    if (this.#zoom < 100) {
      this.#zoom = 100;
    }
    if(this.#zoom <= 250) { this.zoomInButton.disabled = false; }
    if (this.#zoom <= 100) { this.zoomOutButton.disabled = true; }

    window.requestAnimationFrame(() => {
      this.style.setProperty('--location-zoom', this.#zoom !== 100 ? `${this.#zoom}%` : null);
    });
  }
}

customElements.define('location-map', LocationMap);

function zoomButton(inOrOut='in') {
  const element = document.createElement('button');

  element.classList.add('zoom-button');
  element.classList.add(inOrOut);

  element.textContent = inOrOut === 'in' ? '+' : '-';
  return element
}

function actionWrapper() {
  const element = document.createElement('div');

  element.classList.add('action-wrapper');
  return element
}
